﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
//using System.Windows.Forms;

namespace CloudShot.Utils
{
  /// <summary>
  /// Helper class containing Gdi32 API functions
  /// </summary>
  public class GDI32
  {

    public const int SRCCOPY = 0x00CC0020; // BitBlt dwRop parameter

    [DllImport("gdi32.dll")]
    public static extern bool BitBlt(IntPtr hObject, int nXDest, int nYDest,
        int nWidth, int nHeight, IntPtr hObjectSource,
        int nXSrc, int nYSrc, int dwRop);
    [DllImport("gdi32.dll")]
    public static extern IntPtr CreateCompatibleBitmap(IntPtr hDC, int nWidth,
        int nHeight);
    [DllImport("gdi32.dll")]
    public static extern IntPtr CreateCompatibleDC(IntPtr hDC);
    [DllImport("gdi32.dll")]
    public static extern bool DeleteDC(IntPtr hDC);
    [DllImport("gdi32.dll")]
    public static extern bool DeleteObject(IntPtr hObject);
    [DllImport("gdi32.dll")]
    public static extern IntPtr SelectObject(IntPtr hDC, IntPtr hObject);
  }

  /// <summary>
  /// Helper class containing User32 API functions
  /// </summary>
  public class User32
  {
    [StructLayout(LayoutKind.Sequential)]
    public struct RECT
    {
      public int Left;
      public int Top;
      public int Right;
      public int Bottom;
    }

    [DllImport("user32.dll")]
    public static extern IntPtr GetDesktopWindow();
    [DllImport("user32.dll")]
    public static extern IntPtr GetWindowDC(IntPtr hWnd);
    [DllImport("user32.dll")]
    public static extern IntPtr ReleaseDC(IntPtr hWnd, IntPtr hDC);
    [DllImport("user32.dll")]
    public static extern IntPtr GetWindowRect(IntPtr hWnd, ref RECT rect);
    [DllImport("user32.dll")]
    public static extern IntPtr WindowFromPoint(Point point);
    [DllImport("user32.dll", ExactSpelling = true, CharSet = CharSet.Auto)]
    public static extern IntPtr GetParent(IntPtr hWnd);

    public delegate bool EnumDelegate(IntPtr hWnd, int lParam);

    [DllImport("user32.dll", EntryPoint = "EnumDesktopWindows", ExactSpelling = false, CharSet = CharSet.Auto, SetLastError = true)]
    public static extern bool EnumDesktopWindows(IntPtr hDesktop, EnumDelegate lpEnumCallbackFunction, IntPtr lParam);

    [DllImport("user32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    public static extern bool IsWindowVisible(IntPtr hWnd);

    [DllImport("user32.dll")]
    public static extern bool IntersectRect(out RECT lprcDst, [In] ref RECT lprcSrc1, [In] ref RECT lprcSrc2);

    [StructLayout(LayoutKind.Sequential)]
    public struct IconInfo
    {
      public bool fIcon;
      public int xHotspot;
      public int yHotspot;
      public IntPtr hbmMask;
      public IntPtr hbmColor;
    }

    [DllImport("user32.dll")]
    public static extern IntPtr CreateIconIndirect(ref IconInfo icon);

    [DllImport("user32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    public static extern bool GetIconInfo(IntPtr hIcon, ref IconInfo pIconInfo);

    //public static Cursor CreateCursor(Bitmap bmp, int xHotSpot, int yHotSpot)
    //{
    //  User32.IconInfo tmp = new User32.IconInfo();
    //  User32.GetIconInfo(bmp.GetHicon(), ref tmp);
    //  tmp.xHotspot = xHotSpot;
    //  tmp.yHotspot = yHotSpot;
    //  tmp.fIcon = false;
    //  return new Cursor(User32.CreateIconIndirect(ref tmp));
    //}
  }

  public class Gdi32
  {
    /// <summary>
    ///    Performs a bit-block transfer of the color data corresponding to a
    ///    rectangle of pixels from the specified source device context into
    ///    a destination device context.
    /// </summary>
    /// <param name="hdc">Handle to the destination device context.</param>
    /// <param name="nXDest">The leftmost x-coordinate of the destination rectangle (in pixels).</param>
    /// <param name="nYDest">The topmost y-coordinate of the destination rectangle (in pixels).</param>
    /// <param name="nWidth">The width of the source and destination rectangles (in pixels).</param>
    /// <param name="nHeight">The height of the source and the destination rectangles (in pixels).</param>
    /// <param name="hdcSrc">Handle to the source device context.</param>
    /// <param name="nXSrc">The leftmost x-coordinate of the source rectangle (in pixels).</param>
    /// <param name="nYSrc">The topmost y-coordinate of the source rectangle (in pixels).</param>
    /// <param name="dwRop">A raster-operation code.</param>
    /// <returns>
    ///    <c>true</c> if the operation succeeded, <c>false</c> otherwise.
    /// </returns>
    [DllImport("gdi32.dll")]
    [return: MarshalAs(UnmanagedType.Bool)]
    public static extern bool BitBlt(IntPtr hdc, int nXDest, int nYDest, int nWidth, int nHeight, IntPtr hdcSrc, int nXSrc, int nYSrc, TernaryRasterOperations dwRop);

    /// <summary>
    ///     Specifies a raster-operation code. These codes define how the color data for the
    ///     source rectangle is to be combined with the color data for the destination
    ///     rectangle to achieve the final color.
    /// </summary>
    public enum TernaryRasterOperations : uint
    {
      /// <summary>dest = source</summary>
      SRCCOPY = 0x00CC0020,
      /// <summary>dest = source OR dest</summary>
      SRCPAINT = 0x00EE0086,
      /// <summary>dest = source AND dest</summary>
      SRCAND = 0x008800C6,
      /// <summary>dest = source XOR dest</summary>
      SRCINVERT = 0x00660046,
      /// <summary>dest = source AND (NOT dest)</summary>
      SRCERASE = 0x00440328,
      /// <summary>dest = (NOT source)</summary>
      NOTSRCCOPY = 0x00330008,
      /// <summary>dest = (NOT src) AND (NOT dest)</summary>
      NOTSRCERASE = 0x001100A6,
      /// <summary>dest = (source AND pattern)</summary>
      MERGECOPY = 0x00C000CA,
      /// <summary>dest = (NOT source) OR dest</summary>
      MERGEPAINT = 0x00BB0226,
      /// <summary>dest = pattern</summary>
      PATCOPY = 0x00F00021,
      /// <summary>dest = DPSnoo</summary>
      PATPAINT = 0x00FB0A09,
      /// <summary>dest = pattern XOR dest</summary>
      PATINVERT = 0x005A0049,
      /// <summary>dest = (NOT dest)</summary>
      DSTINVERT = 0x00550009,
      /// <summary>dest = BLACK</summary>
      BLACKNESS = 0x00000042,
      /// <summary>dest = WHITE</summary>
      WHITENESS = 0x00FF0062,
      /// <summary>
      /// Capture window as seen on screen.  This includes layered windows 
      /// such as WPF windows with AllowsTransparency="true"
      /// </summary>
      CAPTUREBLT = 0x40000000
    }
  }

}
